import { Construct, CfnOutput } from '@aws-cdk/core';
import { AWSService } from './aws-services';
import { BillingTopicProps } from './billing-topic';
/**
 * @stability stable
 */
export interface AlarmOptions {
    /**
     * Name of the alarm.
     *
     * If you don't specify a name, AWS CloudFormation generates a unique physical ID and uses that ID for the alarm name (recommended).
     *
     * @default Generated name
     * @stability stable
     */
    readonly alarmName?: string;
    /**
     * Description for the alarm.
     *
     * A developer-defined string that can be used to identify this alarm.
     *
     * @default - Not configured
     * @stability stable
     */
    readonly alarmDescription?: string;
    /**
     * Enter the monthly threshold amount in USD that must be exceeded to trigger the alarm e.g. (thresholdAmount: 150).
     *
     * @stability stable
     */
    readonly thresholdAmount: number;
    /**
     * Evaluates the metric every few hours as `EstimatedCharges` metrics are updated every 6 hours.
     *
     * @default Duration.hours(6)
     * @stability stable
     */
    readonly numberOfHours?: number;
}
/**
 * @stability stable
 */
export interface MetricDimensionOptions {
    /**
     * Account which this metric comes from.
     *
     * @default - Not configured.
     * @stability stable
     */
    readonly account?: string;
    /**
     * Region which this metric comes from.
     *
     * @default - Not configured.
     * @stability stable
     */
    readonly region?: string;
    /**
     * The AWS Service to associate the alarm with e.g (AWSService.AMAZON_API_GATEWAY).
     *
     * @default - Not configured.
     * @stability stable
     */
    readonly service?: AWSService | string;
}
/**
 * @stability stable
 */
export interface BillingAlarmProps {
    /**
     * Topic configuration options to configure the SNS topic and email address's that will be used to subscribe to the topic.
     *
     * @stability stable
     */
    readonly topicConfiguration: BillingTopicProps;
    /**
     * Alarm configuration options to configure the billing alarm e.g. (name, description etc.).
     *
     * @stability stable
     */
    readonly alarmConfiguration: AlarmOptions;
    /**
     * Metric dimension options to configure advanced alarm metrics e.g. (link the alarm to a specific account, region or AWS service).
     *
     * @stability stable
     */
    readonly metricDimensions?: MetricDimensionOptions;
}
/**
 * A construct to create an estimated monthly billing alarm associated with an SNS topic, and estimate billing alert notifications via email.
 *
 * @stability stable
 * @example
 *
 * .org'],
 * },
 * alarmConfiguration: {
 * thresholdAmount: 20,
 * }
 * });
 */
export declare class BillingAlarm extends Construct {
    /**
     * @stability stable
     */
    readonly topicArn: CfnOutput;
    /**
     * @stability stable
     */
    constructor(scope: Construct, id: string, props: BillingAlarmProps);
    /**
     * Create a metric.
     *
     * @param props
     * @param amount
     * @returns
     */
    private createMetric;
    /**
     * Create an alarm.
     *
     * @param config
     * @returns
     */
    private createAlarm;
    /**
     * Create a SNS action.
     *
     * @param topic
     * @returns
     */
    private createSnsAction;
}
